#pragma once

#include <stdbool.h>
#include "app_error.h"
#include "fds.h"
#include "nrf_queue.h"
#include "nrf_soc.h"

#define STORAGE_FILE_ID 0x0001

typedef enum {
  STORAGE_CC_KEY           = 0x0001,
  STORAGE_CALIBRATIONS_KEY = 0x0002,
  STORAGE_DOSE_TIMES_KEY   = 0x0003,
} storage_key_t;

typedef enum {
  STORAGE_WRITE_CALIBRATIONS_OPERATION = 0x01,
  STORAGE_READ_CALIBRATIONS_OPERATION = 0x02,
  STORAGE_DELETE_CALIBRATIONS_OPERATION = 0x03,
  STORAGE_WRITE_DOSE_TIMES_OPERATION = 0x04,
  STORAGE_READ_DOSE_TIMES_OPERATION = 0x05,
  STORAGE_DELETE_DOSE_TIMES_OPERATION = 0x06,
  STORAGE_WRITE_COULOMB_COUNTER_OPERATION = 0x07,
  STORAGE_READ_COULOMB_COUNTER_OPERATION = 0x08,
} storage_operation_type_t;

typedef struct {
  float charge;
  uint8_t battery_level;
} storage_coulomb_counter_data_t;

typedef union {
  uint16_t calibration_values[14];
  uint32_t dose_times[14];
  storage_coulomb_counter_data_t coulomb_counter_data;
} storage_operation_data_t;

typedef struct {
  storage_operation_type_t type;
  storage_operation_data_t data;
} storage_operation_t;

ret_code_t storage_init(void);
ret_code_t storage_write_calibrations(uint16_t *p_calibrations);
ret_code_t storage_read_calibrations(uint16_t *p_calibrations);
ret_code_t storage_delete_calibrations(void);
ret_code_t storage_write_dose_times(uint32_t *p_dose_times);
ret_code_t storage_read_dose_times(uint32_t *p_dose_times);
ret_code_t storage_delete_dose_times(void);
ret_code_t storage_write_coulomb_counter(float charge, uint8_t battery_level);
ret_code_t storage_read_coulomb_counter(float *p_charge,
                                        uint8_t *p_battery_level);
void storage_test(void);
